/*
 * control.c
 * MACT library -to- JonoF's Build Port Control System Glue
 * 
 * by Jonathon Fowler
 *
 * Since we don't have the source to the MACT library I've had to
 * concoct some magic to glue its idea of controllers into that of
 * my Build port.
 *
 */
//-------------------------------------------------------------------------
/*
Duke Nukem Copyright (C) 1996, 2003 3D Realms Entertainment

This file is part of Duke Nukem 3D version 1.5 - Atomic Edition

Duke Nukem 3D is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//-------------------------------------------------------------------------

#include "types.h"
#include "keyboard.h"
#include "mouse.h"
#include "control.h"

#include "osd.h"

boolean  CONTROL_RudderEnabled;
boolean  CONTROL_MousePresent;
boolean  CONTROL_JoysPresent[ MaxJoys ];
boolean  CONTROL_MouseEnabled;
boolean  CONTROL_JoystickEnabled;
byte     CONTROL_JoystickPort;
uint32   CONTROL_ButtonState1;
uint32   CONTROL_ButtonHeldState1;
uint32   CONTROL_ButtonState2;
uint32   CONTROL_ButtonHeldState2;

static struct {
	boolean toggle;
	kb_scancode key1, key2;
	int32 mouseb;
	boolean mousedblclk;
} buttons[MAXGAMEBUTTONS];

static fixed axisvalue[6];	// analog_*
static int32 axismapping[6];	// x, y, etc
static int32 axisscaling[6];
static int32 mousesensitivity;

void CONTROL_MapKey( int32 which, kb_scancode key1, kb_scancode key2 )
{
	if (which < 0 || which >= MAXGAMEBUTTONS) return;
	buttons[which].key1 = key1;
	buttons[which].key2 = key2;
}


void CONTROL_MapButton
        (
        int32 whichfunction,
        int32 whichbutton,
        boolean doubleclicked
        )
{
	if (whichfunction < 0 || whichfunction >= MAXGAMEBUTTONS) return;
	if (whichbutton < 0) return;
	buttons[whichfunction].mouseb = whichbutton+1;
	buttons[whichfunction].mousedblclk = doubleclicked;
}


void CONTROL_DefineFlag( int32 which, boolean toggle )
{
	if (which < 0 || which >= MAXGAMEBUTTONS) return;
	buttons[which].toggle = toggle;
}


boolean CONTROL_FlagActive( int32 which )
{
	if (which < 0 || which >= MAXGAMEBUTTONS) return 0;
	return BUTTON(which);
}


void CONTROL_ClearAssignments( void )
{
	int i;

	for (i=0; i<MAXGAMEBUTTONS; i++) {
		buttons[i].key1 = 0;
		buttons[i].key2 = 0;
		buttons[i].mouseb = 0;
		buttons[i].mousedblclk = 0;
		buttons[i].toggle = 0;
	}

	for (i=0; i<6; i++) {
		axismapping[i] = 0;
	}
}


void CONTROL_GetUserInput( UserInput *info )
{
	if (!info) return;

	info->button0 = 0;
	info->button1 = 0;
	info->dir = dir_None;
}


void CONTROL_GetInput( ControlInfo *info )
{
	int i;
	int32 mx,my;
	
	if (!info) return;

	info->dx = 0;
	info->dy = 0;
	info->dz = 0;
	info->dyaw = 0;
	info->dpitch = 0;
	info->droll = 0;

	MOUSE_GetDelta(&mx,&my);
	if (axismapping[0]) axisvalue[axismapping[0]-1] += (mx*mousesensitivity)>>9;
	if (axismapping[1]) axisvalue[axismapping[1]-1] += (my*mousesensitivity)>>7;

	info->dx = axisvalue[analog_strafing];
	info->dy = axisvalue[analog_elevation];
	info->dz = axisvalue[analog_moving];
	info->dyaw = axisvalue[analog_turning];
	info->dpitch = axisvalue[analog_lookingupanddown];
	info->droll = axisvalue[analog_rolling];
	
	for (i=0;i<6;i++) axisvalue[i] = 0;
}


void CONTROL_ClearButton( int32 whichbutton )
{
	uint32 mask;
	
	if (whichbutton < 0 || whichbutton >= MAXGAMEBUTTONS) return;
	if (whichbutton>31) {
		mask = 1ul<<(whichbutton-32);
		CONTROL_ButtonState2 &= ~(0xfffffffful & mask);
	} else {
		mask = 1ul<<whichbutton;
		CONTROL_ButtonState1 &= ~(0xfffffffful & mask);
	}
}


void CONTROL_ClearUserInput( UserInput *info )
{
	if (!info) return;

	info->button0 = 0;
	info->button1 = 0;
	info->dir = dir_None;
}


void CONTROL_WaitRelease( void )
{
}


void CONTROL_Ack( void )
{
}


void CONTROL_CenterJoystick
   (
   void ( *CenterCenter )( void ),
   void ( *UpperLeft )( void ),
   void ( *LowerRight )( void ),
   void ( *CenterThrottle )( void ),
   void ( *CenterRudder )( void )
   )
{
}


int32 CONTROL_GetMouseSensitivity( void )
{
	return mousesensitivity;
}


void CONTROL_SetMouseSensitivity( int32 newsensitivity )
{
	mousesensitivity = newsensitivity;
}


void CONTROL_Startup
   (
   controltype which,
   int32 ( *TimeFunction )( void ),
   int32 ticspersecond
   )
{
	int i;
	
	CONTROL_RudderEnabled = 0;
	CONTROL_MousePresent = 1;
	for (i=0; i<MaxJoys; i++) CONTROL_JoysPresent[i] = 0;
	CONTROL_MouseEnabled = 0;
	CONTROL_JoystickEnabled = 0;
	CONTROL_JoystickPort = 0;
	CONTROL_ButtonState1 = 0;
	CONTROL_ButtonHeldState1 = 0;
	CONTROL_ButtonState2 = 0;
	CONTROL_ButtonHeldState2 = 0;
//	CONTROL_ClearAssignments();

	initinput();
	setkeypresscallback(CONTROL_KeyEvent);
	setmousepresscallback(CONTROL_MouseEvent);

	switch (which) {
		case controltype_keyboardandmouse: MOUSE_Init(); break;
		default: break;
	}
}


void CONTROL_Shutdown( void )
{
	uninitinput();
}


void CONTROL_MapAnalogAxis
   (
   int32 whichaxis,
   int32 whichanalog
   )
{
	axismapping[whichaxis] = whichanalog+1;
}


void CONTROL_MapDigitalAxis
   (
   int32 whichaxis,
   int32 whichfunction,
   int32 direction
   )
{
}


void CONTROL_SetAnalogAxisScale
   (
   int32 whichaxis,
   int32 axisscale
   )
{
	axisscaling[whichaxis] = axisscale;
}


void CONTROL_PrintAxes( void )
{
}


void CONTROL_KeyEvent(kb_scancode scancode, boolean keypressed)
{
	int i;
	uint32 mask;

	for (i=0; i<MAXGAMEBUTTONS; i++) {
		if (buttons[i].key1 == scancode || buttons[i].key2 == scancode) {
			switch (buttons[i].toggle) {
				case 0: if (i>31) {
						mask = 1ul<<(i-32);
						if (keypressed)
							CONTROL_ButtonState2 |= (0xfffffffful & mask);
						else
							CONTROL_ButtonState2 &= ~(0xfffffffful & mask);
					} else {
						mask = 1ul<<i;
						if (keypressed)
							CONTROL_ButtonState1 |= (0xfffffffful & mask);
						else
							CONTROL_ButtonState1 &= ~(0xfffffffful & mask);
					}
					break;
				default:
					if (!keypressed) break;
					if (i>31) {
						mask = 1ul<<(i-32);
						if (CONTROL_ButtonState2 & (0xfffffffful & mask))
							CONTROL_ButtonState2 &= ~(0xfffffffful & mask);
						else
							CONTROL_ButtonState2 |= (0xfffffffful & mask);
					} else {
						mask = 1ul<<i;
						if (CONTROL_ButtonState1 & (0xfffffffful & mask))
							CONTROL_ButtonState1 &= ~(0xfffffffful & mask);
						else
							CONTROL_ButtonState1 |= (0xfffffffful & mask);
					}
					break;
			}
		}
	}
}

void CONTROL_MouseEvent(int32 button, boolean keypressed)
{
	int i;
	uint32 mask;

	for (i=0; i<MAXGAMEBUTTONS; i++) {
		if (buttons[i].mouseb == button) {
			switch (buttons[i].toggle) {
				case 0: if (i>31) {
						mask = 1ul<<(i-32);
						if (keypressed)
							CONTROL_ButtonState2 |= (0xfffffffful & mask);
						else
							CONTROL_ButtonState2 &= ~(0xfffffffful & mask);
					} else {
						mask = 1ul<<i;
						if (keypressed)
							CONTROL_ButtonState1 |= (0xfffffffful & mask);
						else
							CONTROL_ButtonState1 &= ~(0xfffffffful & mask);
					}
					break;
				default:
					if (!keypressed) break;
					if (i>31) {
						mask = 1ul<<(i-32);
						if (CONTROL_ButtonState2 & (0xfffffffful & mask))
							CONTROL_ButtonState2 &= ~(0xfffffffful & mask);
						else
							CONTROL_ButtonState2 |= (0xfffffffful & mask);
					} else {
						mask = 1ul<<i;
						if (CONTROL_ButtonState1 & (0xfffffffful & mask))
							CONTROL_ButtonState1 &= ~(0xfffffffful & mask);
						else
							CONTROL_ButtonState1 |= (0xfffffffful & mask);
					}
					break;
			}
		}
	}
}


